@@ -1,4 +1,4 @@
-/*  $Id: his.c 10305 2018-12-02 14:21:56Z iulius $
+/*  $Id: his.c,v 1.16.2.1 2003/05/19 02:00:06 rra Exp $
 **
 **  API to history routines 
 **
@@ -39,19 +39,15 @@
 
 #include "config.h"
 #include "clibrary.h"
+#include "portable/time.h"
 #include <errno.h>
 #include <syslog.h>
 
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#endif
-#include <time.h>
-
 #include "inn/history.h"
 #include "inn/messages.h"
 #include "inn/timer.h"
-#include "inn/libinn.h"
-#include "inn/storage.h"
+#include "libinn.h"
+#include "storage.h"
 
 #include "hisinterface.h"
 #include "hismethods.h"
@@ -72,7 +68,7 @@ struct history {
 
 enum HISRESULT {HIScachehit, HIScachemiss, HIScachedne};
 
-static const struct histstats nullhist = { 0, 0, 0, 0 };
+static const struct histstats nullhist = {0};
 
 /*
 ** Put an entry into the history cache 
@@ -280,14 +276,14 @@ HISwrite(struct history *h, const char *key, time_t arrived,
 }
 
 bool
-HISremember(struct history *h, const char *key, time_t arrived, time_t posted)
+HISremember(struct history *h, const char *key, time_t arrived)
 {
     bool r;
 
     if (his_checknull(h))
 	return false;
     TMRstart(TMR_HISWRITE);
-    r = (*h->methods->remember)(h->sub, key, arrived, posted);
+    r = (*h->methods->remember)(h->sub, key, arrived);
     if (r == true) {
 	HASH hash;
 
@@ -426,7 +422,7 @@ void HISlogto(const char *s) {
 
    HISlogclose();
    if ((HISfdlog = Fopen(s, "a", INND_HISLOG)) == NULL)
-       syswarn("cant open %s", s);
+       syslog(L_FATAL, "cant open %s %m", s);
    /* initialize our counters */
    for (i = 0; i < S_HIS_MAX; i++) {
        HISstat_start[i].tv_sec = 0;
@@ -438,24 +434,22 @@ void HISlogto(const char *s) {
 }
 
 void
-his_logger(const char *s, int code)
+his_logger(char *s, int code)
 {
     struct timeval tv;
     struct tm *tm;
-    time_t t;
 
     if (HISfdlog == NULL) /* do nothing unless HISlogto() has been called */
 	return;
 
     gettimeofday(&tv, NULL);
-    t = tv.tv_sec;
-    tm = localtime(&t);
+    tm = localtime((const time_t *)&(tv.tv_sec));
     if (HISstat_start[code].tv_sec != 0) {
 	fprintf(HISfdlog, "%d/%d/%d %02d:%02d:%02d.%06d: [%d] %s (%.6f)\n",
 		tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour,
-		tm->tm_min, tm->tm_sec, (int)tv.tv_usec, code, s, (double) tv.tv_sec +
-        (double) tv.tv_usec / 1000000 - (double) HISstat_start[code].tv_sec -
-        (double) HISstat_start[code].tv_usec / 1000000);
+		tm->tm_min, tm->tm_sec, (int)tv.tv_usec, code, s, (float) tv.tv_sec +
+		(float) tv.tv_usec / 1000000 - (float) HISstat_start[code].tv_sec -
+		(float) HISstat_start[code].tv_usec / 1000000);
 	if (tv.tv_usec < HISstat_start[code].tv_usec) {
 	    HISstat_total[code].tv_sec++;
 	    HISstat_total[code].tv_usec +=
