@@ -1,4 +1,4 @@
-/*  $Revision: 1.4 $
+/*  $Revision: 1.5 $
 **
 **  Data structures, functions and cetera used for config file parsing.
 */
@@ -12,6 +12,8 @@ typedef struct {
     char *buf;
     unsigned int sbuf;
     int lineno;
+    int array_len;
+    char **array;
     char *filename;
 } CONFFILE;
 
