@@ -1,45 +1,46 @@
-/*  $Id: lc.c,v 1.10 1999/11/21 00:15:44 rra Exp $
+/*  $Id: lc.c,v 1.22 2003/01/19 19:58:21 rra Exp $
 **
 **  Routines for the local connect channel.  Create a Unix-domain stream
 **  socket that processes on the local server connect to.  Once the
 **  connection is set up, we speak NNTP.  The connect channel is used only
 **  by rnews to feed in articles from the UUCP sites.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
-#include "clibrary.h"
-#include "innd.h"
-#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
-#include <sys/un.h>
 
-STATIC char	*LCpath = NULL;
-STATIC CHANNEL	*LCchan;
+#include "config.h"
+#include "clibrary.h"
+
+#include "inn/innconf.h"
+#include "innd.h"
+
+
+#if HAVE_UNIX_DOMAIN_SOCKETS
+# include <sys/un.h>
+
+static char	*LCpath = NULL;
+static CHANNEL	*LCchan;
 
 
 /*
 **  Read function.  Accept the connection and create an NNTP channel.
 */
-STATIC FUNCTYPE
-LCreader(cp)
-    CHANNEL	*cp;
+static void
+LCreader(CHANNEL *cp)
 {
     int		fd;
     CHANNEL	*new;
 
     if (cp != LCchan) {
-	syslog(L_ERROR, "%s internal LCreader wrong channel 0x%x not 0x%x",
-	    LogName, cp, LCchan);
+	syslog(L_ERROR, "%s internal LCreader wrong channel 0x%p not 0x%p",
+	    LogName, (void *)cp, (void *)LCchan);
 	return;
     }
 
-    if ((fd = accept(cp->fd, (struct sockaddr *)NULL, (ARGTYPE *)NULL)) < 0) {
+    if ((fd = accept(cp->fd, NULL, NULL)) < 0) {
 	syslog(L_ERROR, "%s cant accept CCreader %m", LogName);
 	return;
     }
-    if ((new = NCcreate(fd, FALSE, TRUE)) != NULL) {
-	new->Address.s_addr = MyAddress.s_addr;
+    if ((new = NCcreate(fd, false, true)) != NULL) {
+	memset( &new->Address, 0, sizeof( new->Address ) );
 	syslog(L_NOTICE, "%s connected %d", "localhost", new->fd);
 	NCwritereply(new, (char *)NCgreeting);
     }
@@ -49,26 +50,28 @@ LCreader(cp)
 /*
 **  Write-done function.  Shouldn't happen.
 */
-STATIC FUNCTYPE
-LCwritedone()
+static void
+LCwritedone(CHANNEL *unused)
 {
+    unused = unused;		/* ARGSUSED */
     syslog(L_ERROR, "%s internal LCwritedone", LogName);
 }
-#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
+
+#endif /* HAVE_UNIX_DOMAIN_SOCKETS */
 
 
 /*
 **  Create the channel.
 */
 void
-LCsetup()
+LCsetup(void)
 {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     int			i;
     struct sockaddr_un	server;
 
     if (LCpath == NULL)
-	LCpath = COPY(cpcatpath(innconf->pathrun, _PATH_NNTPCONNECT));
+	LCpath = concatpath(innconf->pathrun, _PATH_NNTPCONNECT);
     /* Remove old detritus. */
     if (unlink(LCpath) < 0 && errno != ENOENT) {
 	syslog(L_FATAL, "%s cant unlink %s %m", LogName, LCpath);
@@ -80,9 +83,9 @@ LCsetup()
 	syslog(L_FATAL, "%s cant socket %s %m", LogName, LCpath);
 	exit(1);
     }
-    (void)memset((POINTER)&server, 0, sizeof server);
+    memset(&server, 0, sizeof server);
     server.sun_family = AF_UNIX;
-    (void)strcpy(server.sun_path, LCpath);
+    strlcpy(server.sun_path, LCpath, sizeof(server.sun_path));
     if (bind(i, (struct sockaddr *) &server, SUN_LEN(&server)) < 0) {
 	syslog(L_FATAL, "%s cant bind %s %m", LogName, LCpath);
 	exit(1);
@@ -104,13 +107,13 @@ LCsetup()
 **  Cleanly shut down the channel.
 */
 void
-LCclose()
+LCclose(void)
 {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     CHANclose(LCchan, CHANname(LCchan));
     LCchan = NULL;
     if (unlink(LCpath) < 0)
 	syslog(L_ERROR, "%s cant unlink %s %m", LogName, LCpath);
-    DISPOSE(LCpath);
+    free(LCpath);
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 }
