@@ -1,4 +1,4 @@
-/*  $Id: rc.c,v 1.83 2003/01/19 19:58:21 rra Exp $
+/*  $Id: rc.c,v 1.83.2.1 2003/05/11 19:40:28 rra Exp $
 **
 **  Routines for the remote connect channel.  Create an Internet stream
 **  socket that processes connect to.  If the incoming site is not one of
@@ -7,7 +7,7 @@
 */
 #include "config.h"
 #include "clibrary.h"
-#include <arpa/inet.h>
+#include "portable/socket.h"
 #include <errno.h>
 #include <netdb.h>
 
@@ -1658,7 +1658,7 @@ RCwritelist(char *filename)
         if (*p == '/')
 	   q = p + 1;
 
-    fprintf (F, "##  $Revision: 1.83 $\n");
+    fprintf (F, "##  $Revision: 1.83.2.1 $\n");
     fprintf (F, "##  %s - names and addresses that feed us news\n", q);
     free(r);
     fprintf (F, "##\n\n");
