@@ -1,10 +1,14 @@
-/*  $Id: status.c,v 1.22 2000/04/03 06:57:21 rra Exp $
+/*  $Id: status.c,v 1.34.2.2 2003/05/31 19:48:26 rra Exp $
 **
 **  Periodic status reporting.
 */
+#include "config.h"
 #include "clibrary.h"
+#include "portable/socket.h"
+
+#include "inn/innconf.h"
 #include "innd.h"
-#include <arpa/inet.h>
+#include "innperl.h"
 
 #define MIN_REFRESH   60  /* 1 min */
 #define HTML_STATUS
@@ -15,43 +19,44 @@
 #endif
 
 typedef struct _STATUS {
-    char           name[SMBUF];
-    char           ip_addr[15];
-    BOOL           can_stream;
-    unsigned short activeCxn;
-    unsigned short sleepingCxns;
-    unsigned long  seconds;
-    unsigned long  accepted;
-    unsigned long  refused;
-    unsigned long  rejected;
-    unsigned long  Duplicate;
-    unsigned long  Unwanted_u;
-    unsigned long  Unwanted_d;
-    unsigned long  Unwanted_g;
-    unsigned long  Unwanted_s;
-    unsigned long  Unwanted_f;
-    float          Size;
-    float          DuplicateSize;
-    unsigned long  Check;
-    unsigned long  Check_send;
-    unsigned long  Check_deferred;
-    unsigned long  Check_got;
-    unsigned long  Check_cybercan;
-    unsigned long  Takethis;
-    unsigned long  Takethis_Ok;
-    unsigned long  Takethis_Err;
-    unsigned long  Ihave;
-    unsigned long  Ihave_Duplicate;
-    unsigned long  Ihave_Deferred;
-    unsigned long  Ihave_SendIt;
-    unsigned long  Ihave_Cybercan;
+    char		name[SMBUF];
+    char		ip_addr[64];
+    bool		can_stream;
+    unsigned short	activeCxn;
+    unsigned short	sleepingCxns;
+    time_t		seconds;
+    unsigned long	accepted;
+    unsigned long	refused;
+    unsigned long	rejected;
+    unsigned long	Duplicate;
+    unsigned long	Unwanted_u;
+    unsigned long	Unwanted_d;
+    unsigned long	Unwanted_g;
+    unsigned long	Unwanted_s;
+    unsigned long	Unwanted_f;
+    float		Size;
+    float		DuplicateSize;
+    unsigned long	Check;
+    unsigned long	Check_send;
+    unsigned long	Check_deferred;
+    unsigned long	Check_got;
+    unsigned long	Check_cybercan;
+    unsigned long	Takethis;
+    unsigned long	Takethis_Ok;
+    unsigned long	Takethis_Err;
+    unsigned long	Ihave;
+    unsigned long	Ihave_Duplicate;
+    unsigned long	Ihave_Deferred;
+    unsigned long	Ihave_SendIt;
+    unsigned long	Ihave_Cybercan;
     struct _STATUS *next;
 } STATUS;
 
 static unsigned STATUSlast_time;
 char            start_time[50];
 
-static unsigned STATUSgettime(void)
+static unsigned
+STATUSgettime(void)
 {
   static int	    	init = 0;
   static struct timeval	start_tv;
@@ -66,16 +71,18 @@ static unsigned STATUSgettime(void)
 	 (tv.tv_usec - start_tv.tv_usec) / 1000);
 }
 
-void STATUSinit(void)
+void
+STATUSinit(void)
 {
   time_t now;
   
   STATUSlast_time = STATUSgettime();	/* First invocation */
   now = time (NULL) ;
-  strcpy (start_time, ctime (&now)) ;
+  strlcpy(start_time, ctime(&now), sizeof(start_time));
 }
 
-char *PrettySize (float size, char *str)
+static char *
+PrettySize(float size, char *str)
 {
   if (size > 1073741824) /* 1024*1024*1024 */
     sprintf (str, "%.1fGb", size / 1073741824.);
@@ -87,42 +94,41 @@ char *PrettySize (float size, char *str)
   return (str);
 }
 
-static void STATUSsummary(void)
+static void
+STATUSsummary(void)
 {
-  FILE             *F;
-  int               i, j;
-  register CHANNEL *cp;
-  int               activeCxn = 0;
-  int               sleepingCxns = 0;
-  unsigned long     seconds = 0;
-  unsigned long     duplicate = 0;
-  unsigned long     offered;
-  unsigned long     accepted = 0;
-  unsigned long     refused = 0;
-  unsigned long     rejected = 0;
-  float             size = 0;
-  float             DuplicateSize = 0;
-  int               peers = 0;
-  char              TempString [SMBUF];
-  STATUS            *head, *status, *tmp;
-  char              str[9];
-  time_t            now;
-#if defined(DO_PERL)
-  extern int        PerlFilterActive;
-#endif /* defined(DO_PERL) */
+  FILE			*F;
+  int			i, j;
+  CHANNEL               *cp;
+  int			activeCxn = 0;
+  int			sleepingCxns = 0;
+  time_t		seconds = 0;
+  unsigned long		duplicate = 0;
+  unsigned long		offered;
+  unsigned long		accepted = 0;
+  unsigned long		refused = 0;
+  unsigned long		rejected = 0;
+  float			size = 0;
+  float			DuplicateSize = 0;
+  int			peers = 0;
+  char                  TempString[SMBUF];
+  char			*path;
+  STATUS		*head, *status, *tmp;
+  char			str[9];
+  time_t		now;
  
 #if defined(HTML_STATUS)
-  sprintf (TempString, "%s/%s", innconf->pathhttp, STATUS_FILE);
+  path = concatpath(innconf->pathhttp, STATUS_FILE);
 #else
-  sprintf (TempString, "%s/%s", innconf->pathlog, STATUS_FILE);
+  path = concatpath(innconf->pathlog, STATUS_FILE);
 #endif
-  if ((F = Fopen(TempString, "w", TEMPORARYOPEN)) == NULL)
+  if ((F = Fopen(path, "w", TEMPORARYOPEN)) == NULL)
     return;
 
 #if defined(HTML_STATUS)
   /* HTML Header */
 
-  fprintf (F,"<HTML>\n<HEAD>\n<META HTTP-EQUIV=\"Refresh\" CONTENT=\"%d;\">\n",
+  fprintf (F,"<HTML>\n<HEAD>\n<META HTTP-EQUIV=\"Refresh\" CONTENT=\"%ld;\">\n",
 	   innconf->status < MIN_REFRESH ? MIN_REFRESH : innconf->status);
   fprintf (F, "<TITLE>%s: incoming feeds</TITLE>\n", innconf->pathhost);
   fprintf (F, "</HEAD>\n<BODY>\n<PRE>\n") ;
@@ -134,16 +140,20 @@ static void STATUSsummary(void)
   tmp = head = NULL;
   for (i = 0; (cp = CHANiter(&i, CTnntp)) != NULL; ) {
     j = 0;
-    strcpy(TempString, cp->Address.s_addr == 0 ? "localhost" : RChostname(cp));
+    strlcpy(TempString,
+	    cp->Address.ss_family == 0 ? "localhost" : RChostname(cp),
+            sizeof(TempString));
     for (status = head ; status != NULL ; status = status->next) {
 	if (strcmp(TempString, status->name) == 0)
 	    break;
     }
     if (status == NULL) {
-      status = NEW(STATUS, 1);
+      status = xmalloc(sizeof(STATUS));
       peers++;                                              /* a new peer */
-      strcpy (status->name, TempString);                         /* name */
-      strcpy (status->ip_addr, inet_ntoa(cp->Address));    /* ip address */
+      strlcpy(status->name, TempString, sizeof(status->name));
+      strlcpy(status->ip_addr,
+              sprint_sockaddr((struct sockaddr *)&cp->Address),
+              sizeof(status->ip_addr));
       status->can_stream = cp->Streaming;
       status->seconds = status->Size = status->DuplicateSize = 0;
       status->Ihave = status->Ihave_Duplicate =
@@ -211,7 +221,7 @@ static void STATUSsummary(void)
 
   /* Header */
   now = time (NULL);
-  strcpy (TempString, ctime (&now));
+  strlcpy (TempString, ctime (&now), sizeof(TempString));
   fprintf (F, "Updated: %s", TempString);
   fprintf (F, "(peers: %d, active-cxns: %d, sleeping-cxns: %d)\n\n",
 	   peers, activeCxn, sleepingCxns);
@@ -229,13 +239,13 @@ static void STATUSsummary(void)
   fprintf (F, "              Largest Article: %ld bytes\n", innconf->maxartsize);
   fprintf (F, "     Max Incoming connections: ");
   if (innconf->maxconnections)
-    fprintf (F, "%d\n", innconf->maxconnections);
+    fprintf (F, "%ld\n", innconf->maxconnections);
   else
     fprintf (F, "unlimited\n");
   fprintf (F, "      Max Outgoing file feeds: %d\n", MaxOutgoing);
   fprintf (F, "                       Cutoff: ");
   if (innconf->artcutoff)
-    fprintf (F, "%d days\n", (int) (innconf->artcutoff / 3600L / 24L));
+    fprintf (F, "%ld days\n", innconf->artcutoff);
   else
     fprintf (F, "none\n");
   fprintf (F, "               Timeout period: %ld seconds\n",
@@ -258,7 +268,7 @@ static void STATUSsummary(void)
 
   /* Global values */
   fprintf (F, "global (process)\n");
-  fprintf (F, "         seconds: %ld\n", seconds);
+  fprintf (F, "         seconds: %ld\n", (long) seconds);
   offered = accepted + refused + rejected;
   fprintf (F, "         offered: %-9ld\n", offered);
   if (!offered) offered = 1; /* to avoid division by zero */
@@ -279,7 +289,7 @@ static void STATUSsummary(void)
   /* Incoming Feeds */
   for (status = head ; status != NULL ;) {
     fprintf (F, "%s\n",                      status->name);
-    fprintf (F, "    seconds: %-7ld  ",      status->seconds);
+    fprintf (F, "    seconds: %-7ld  ",      (long) status->seconds);
     fprintf (F, "      duplicates: %-7ld ",  status->Duplicate);
     fprintf (F, "    ip address: %s\n",      status->ip_addr);
     fprintf (F, "    offered: %-7ld  ",
@@ -319,7 +329,7 @@ static void STATUSsummary(void)
     }
     fputc ('\n', F) ;
     tmp = status->next;
-    DISPOSE(status);
+    free(status);
     status = tmp;
   }
 
@@ -328,10 +338,11 @@ static void STATUSsummary(void)
   fprintf (F,"</PRE>\n</BODY>\n</HTML>\n");
 #endif /* defined(HTML_STATUS) */
 
-  (void)Fclose(F);
+  Fclose(F);
 }
 
-void STATUSmainloophook(void)
+void
+STATUSmainloophook(void)
 {
   unsigned now;
     
@@ -339,9 +350,8 @@ void STATUSmainloophook(void)
     return;
   now = STATUSgettime();
   
-  if (now - STATUSlast_time > (innconf->status * 1000)) {
+  if (now - STATUSlast_time > (unsigned)(innconf->status * 1000)) {
     STATUSsummary();
     STATUSlast_time = now;
   }
 }
-
