@@ -1,53 +1,76 @@
-/* $Revision: 1.9 $
+/*  $Id: conffile.c,v 1.22 2003/01/19 01:15:33 rra Exp $
 **
-** Routines for reading in incoming.conf-style config files.
+**  Routines for reading in incoming.conf-style config files.
 */
+
 #include "config.h"
 #include "clibrary.h"
 #include "conffile.h"
 #include "libinn.h"
-#include "macros.h"
+
+static int getconfline(CONFFILE *F, char *buffer, int length) {
+  if (F->f) {
+    fgets(buffer, length, F->f);
+    if (ferror(F->f)) {
+      return 1;
+    }
+  } else if (F->array) {
+    strlcpy(buffer, F->array[F->lineno], F->sbuf);
+  }
+  F->lineno++;
+  if (strlen (F->buf) == F->sbuf) {
+    return 1; /* Line too long */
+  } else {
+    return 0;
+  }
+}
+
+static int cfeof(CONFFILE *F) {
+  if (F->f) {
+    return feof(F->f);
+  } else if (F->array) {
+    return (F->lineno == F->array_len);
+  } else {
+    return 1;
+  }
+}
 
 static char *CONFgetword(CONFFILE *F)
 {
-  register char *p;
-  register char *s;
-  register char *t;
-  char          *word;
-  register BOOL flag;
+  char *p;
+  char *s;
+  char *t;
+  char *word;
+  bool flag;
 
   if (!F) return (NULL);	/* No conf file */
   if (!F->buf || !F->buf[0]) {
-    if (feof (F->f)) return (NULL);
+    if (cfeof (F)) return (NULL);
     if (!F->buf) {
       F->sbuf = BIG_BUFFER;
-      F->buf = NEW(char, F->sbuf);
+      F->buf = xmalloc(F->sbuf);
     }
-    fgets(F->buf, F->sbuf, F->f);
-    F->lineno++;
-    if (strlen (F->buf) == F->sbuf)
+    if (getconfline(F, F->buf, F->sbuf) != 0)
       return (NULL); /* Line too long */
   }
   do {
      /* Ignore blank and comment lines. */
      if ((p = strchr(F->buf, '\n')) != NULL)
        *p = '\0';
-     if ((p = strchr(F->buf, COMMENT_CHAR)) != NULL) {
+     if ((p = strchr(F->buf, '#')) != NULL) {
        if (p == F->buf || (p > F->buf && *(p - 1) != '\\'))
            *p = '\0';
      }
      for (p = F->buf; *p == ' ' || *p == '\t' ; p++);
-     flag = TRUE;
-     if (*p == '\0' && !feof (F->f)) {
-       flag = FALSE;
-       fgets(F->buf, F->sbuf, F->f);
-       F->lineno++;
-       if (strlen (F->buf) == F->sbuf)
+     flag = true;
+     if (*p == '\0' && !cfeof(F)) {
+       flag = false;
+       if (getconfline(F, F->buf, F->sbuf))
          return (NULL); /* Line too long */
        continue;
      }
      break;
-  } while (!feof (F->f) || !flag);
+  } while (!cfeof(F) || !flag);
 
   if (*p == '"') { /* double quoted string ? */
     p++;
@@ -56,16 +79,14 @@ static char *CONFgetword(CONFFILE *F)
              *t != '\0'; t++);
       if (*t == '\0') {
         *t++ = '\n';
-        fgets(t, F->sbuf - strlen (F->buf), F->f);
-	F->lineno++;
-        if (strlen (F->buf) == F->sbuf)
+        if (getconfline(F, t, F->sbuf - strlen(F->buf)))
           return (NULL); /* Line too long */
         if ((s = strchr(t, '\n')) != NULL)
           *s = '\0';
       }
       else 
         break;
-    } while (!feof (F->f));
+    } while (!cfeof(F));
     *t++ = '\0';
   }
   else {
@@ -73,8 +94,8 @@ static char *CONFgetword(CONFFILE *F)
     if (*t != '\0')
       *t++ = '\0';
   }
-  if (*p == '\0' && feof (F->f)) return (NULL);
-  word = COPY (p);
+  if (*p == '\0' && cfeof(F)) return (NULL);
+  word = xstrdup (p);
   for (p = F->buf; *t != '\0'; t++)
     *p++ = *t;
   *p = '\0';
@@ -90,16 +111,17 @@ CONFFILE *CONFfopen(char *filename)
   f = fopen(filename, "r");
   if (!f)
     return(0);
-  ret = NEW(CONFFILE, 1);
+  ret = xmalloc(sizeof(CONFFILE));
   if (!ret) {
     fclose(f);
     return(0);
   }
-  ret->filename = COPY(filename);
+  ret->filename = xstrdup(filename);
   ret->buf = 0;
   ret->sbuf = 0;
   ret->lineno = 0;
   ret->f = f;
+  ret->array = NULL;
   return(ret);
 }
 
@@ -108,10 +130,10 @@ void CONFfclose(CONFFILE *f)
   if (!f) return;		/* No conf file */
   fclose(f->f);
   if (f->buf)
-    DISPOSE(f->buf);
+    free(f->buf);
   if (f->filename)
-    DISPOSE(f->filename);
-  DISPOSE(f);
+    free(f->filename);
+  free(f);
 }
 
 CONFTOKEN *CONFgettoken(CONFTOKEN *toklist, CONFFILE *file)
@@ -121,7 +143,7 @@ CONFTOKEN *CONFgettoken(CONFTOKEN *toklist, CONFFILE *file)
   int i;
 
   if (ret.name) {
-    DISPOSE(ret.name);
+    free(ret.name);
     ret.name = 0;
   }
   word = CONFgetword(file);
@@ -129,8 +151,8 @@ CONFTOKEN *CONFgettoken(CONFTOKEN *toklist, CONFFILE *file)
     return(0);
   if (toklist) {
     for (i = 0; toklist[i].type; i++) {
-       if (EQ(word, toklist[i].name)) {
-         DISPOSE(word);
+       if (strcmp(word, toklist[i].name) == 0) {
+         free(word);
          return(&toklist[i]);
        }
     }
