@@ -1,4 +1,4 @@
-/*  $Id: daemonize.c,v 1.4 2002/08/27 08:01:56 alexk Exp $
+/*  $Id: daemonize.c,v 1.4.2.1 2003/07/12 19:13:49 rra Exp $
 **
 **  Become a long-running daemon.
 **
@@ -34,18 +34,21 @@ daemonize(const char *path)
     else if (status > 0)
         _exit(0);
 
-#ifdef TIOCNOTTY
+    /* setsid() should take care of disassociating from the controlling
+       terminal, and FreeBSD at least doesn't like TIOCNOTTY if you don't
+       already have a controlling terminal.  So only use the older TIOCNOTTY
+       method if setsid() isn't available. */
+#if HAVE_SETSID
+    if (setsid() < 0)
+        syswarn("cant become session leader");
+#elif defined(TIOCNOTTY)
     fd = open("/dev/tty", O_RDWR);
     if (fd >= 0) {
         if (ioctl(fd, TIOCNOTTY, NULL) < 0)
             syswarn("cant disassociate from the terminal");
         close(fd);
     }
-#endif /* TIOCNOTTY */
-
-#if HAVE_SETSID
-    setsid();
-#endif
+#endif /* defined(TIOCNOTTY) */
 
     if (chdir(path) < 0)
         syswarn("cant chdir to %s", path);
