@@ -1,66 +1,28 @@
-/* $Id: snprintf.c 10396 2020-11-12 20:19:41Z iulius $
- *
- * Replacement for a missing snprintf or vsnprintf.
- *
- * The following implementation of snprintf was taken mostly verbatim from
- * <http://www.fiction.net/blong/programs/>; it is the version of snprintf
- * used in Mutt.  A possibly newer version is used in wget, found at
- * <https://github.com/wertarbyte/wget/blob/master/src/snprintf.c>.
- *
- * Please do not reformat or otherwise change this file more than necessary so
- * that later merges with the original source are easy.  Bug fixes and
- * improvements should be sent back to the original author.
- *
- * The canonical version of this file was maintained (until 2020) in the
- * rra-c-util package, which can be found at
- * <https://www.eyrie.org/~eagle/software/rra-c-util/>.
- */
+/*  $Id: snprintf.c,v 1.9 2002/08/29 04:07:22 rra Exp $
+**
+**  Replacement for a missing snprintf or vsnprintf.
+**
+**  The following implementation of snprintf was taken mostly verbatim from
+**  <http://www.fiction.net/~blong/programs/>; it is the version of snprintf
+**  used in Mutt.
+**
+**  Please do not reformat or otherwise change this file more than
+**  necessary so that later merges with the original source are easy.
+**  Bug fixes and improvements should be sent back to the original author.
+*/
 
-/*
- * If we're running the test suite, rename snprintf and vsnprintf to avoid
- * conflicts with the system version.
- */
+/* If we're running the test suite, rename snprintf and vsnprintf to avoid
+   conflicts with the system version. */
 #if TESTING
-# undef snprintf
-# undef vsnprintf
 # define snprintf test_snprintf
 # define vsnprintf test_vsnprintf
 #endif
 
-/*
- * __attribute__ is available in gcc 2.5 and later, but only with gcc 2.7
- * could you use the __format__ form of the attributes, which is what we use
- * (to avoid confusion with other macros).
- */
-#ifndef __attribute__
-#    if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#        define __attribute__(spec) /* empty */
-#    endif
-#endif
-
-/*
- * Older Clang doesn't support __attribute__((fallthrough)) properly and
- * complains about the empty statement that it is decorating.  Suppress that
- * warning.  Also suppress warnings about unknown attributes to handle older
- * Clang versions.
- */
-#if !defined(__attribute__) && (defined(__llvm__) || defined(__clang__))
-#    pragma GCC diagnostic ignored "-Wattributes"
-#    pragma GCC diagnostic ignored "-Wmissing-declarations"
-#endif
-
-/* Specific to rra-c-util, but only when debugging is enabled. */
-#ifdef DEBUG_SNPRINTF
-# include "inn/messages.h"
-#endif
-
 /*
  * Copyright Patrick Powell 1995
  * This code is based on code written by Patrick Powell (papowell@astart.com)
  * It may be used for any purpose as long as this notice remains intact
  * on all source code distributions
- *
- * There is no SPDX-License-Identifier registered for this license.
  */
 
 /**************************************************************
@@ -88,7 +50,7 @@
  *    probably requires libm on most operating systems.  Don't yet
  *    support the exponent (e,E) and sigfig (g,G).  Also, fmtint()
  *    was pretty badly broken, it just wasn't being exercised in ways
- *    which showed it, so that's been fixed.  Also, formatted the code
+ *    which showed it, so that's been fixed.  Also, formated the code
  *    to mutt conventions, and removed dead code left over from the
  *    original.  Also, there is now a builtin-test, just compile with:
  *           gcc -DTEST_SNPRINTF -o snprintf snprintf.c -lm
@@ -107,41 +69,18 @@
  *    fixed handling of %.0f
  *    added test for HAVE_LONG_DOUBLE
  *
- *  Russ Allbery <eagle@eyrie.org> 2000-08-26
+ *  Russ Allbery <rra@stanford.edu> 2000-08-26
  *    fixed return value to comply with C99
  *    fixed handling of snprintf(NULL, ...)
- *    added explicit casts for double to long long int conversion
- *    fixed various warnings with GCC 7
- *    fixed various warnings with Clang
  *
- *  Hrvoje Niksic <hniksic@xemacs.org> 2000-11-04
- *    include <config.h> instead of "config.h".
- *    moved TEST_SNPRINTF stuff out of HAVE_SNPRINTF ifdef.
+ *  Hrvoje Niksic <hniksic@arsdigita.com> 2000-11-04
  *    include <stdio.h> for NULL.
- *    added support and test cases for long long.
+ *    added support for long long.
  *    don't declare argument types to (v)snprintf if stdarg is not used.
- *    use int instead of short int as 2nd arg to va_arg.
- *
- *  alexk (INN) 2002-08-21
- *    use LLONG in fmtfp to handle more characters during floating
- *    point conversion.
- *
- *  herb (Samba) 2002-12-19
- *    actually print args for %g and %e
- *
- *  Hrvoje Niksic <hniksic@xemacs.org> 2005-04-15
- *    use the PARAMS macro to handle prototypes.
- *    write function definitions in the ansi2knr-friendly way.
- *    if string precision is specified, don't read VALUE past it.
- *    fix bug in fmtfp that caused 0.01 to be printed as 0.1.
- *    don't include <ctype.h> because none of it is used.
- *    interpret precision as number of significant digits with %g
- *    omit trailing decimal zeros with %g
  *
  **************************************************************/
 
 #include "config.h"
-
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
@@ -153,11 +92,19 @@
 /* varargs declarations: */
 
 #include <stdarg.h>
+#define HAVE_STDARGS    /* let's hope that works everywhere (mj) */
+#define VA_LOCAL_DECL   va_list ap
+#define VA_START(f)     va_start(ap, f)
+#define VA_SHIFT(v,t)  ;   /* no-op for ANSI */
+#define VA_END          va_end(ap)
 
-/* Assume all compilers support long double, per Autoconf documentation. */
+#ifdef HAVE_LONG_DOUBLE
 #define LDOUBLE long double
+#else
+#define LDOUBLE double
+#endif
 
-#ifdef HAVE_LONG_LONG_INT
+#ifdef HAVE_LONG_LONG
 # define LLONG long long
 #else
 # define LLONG long
@@ -169,7 +116,7 @@ int vsnprintf (char *str, size_t count, const char *fmt, va_list arg);
 static int dopr (char *buffer, size_t maxlen, const char *format, 
                  va_list args);
 static int fmtstr (char *buffer, size_t *currlen, size_t maxlen,
-		   const char *value, int flags, int min, int max);
+		   char *value, int flags, int min, int max);
 static int fmtint (char *buffer, size_t *currlen, size_t maxlen,
 		   LLONG value, int base, int min, int max, int flags);
 static int fmtfp (char *buffer, size_t *currlen, size_t maxlen,
@@ -199,7 +146,6 @@ static int dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c );
 #define DP_F_ZERO  	(1 << 4)
 #define DP_F_UP    	(1 << 5)
 #define DP_F_UNSIGNED 	(1 << 6)
-#define DP_F_FP_G 	(1 << 7)
 
 /* Conversion Flags */
 #define DP_C_SHORT   1
@@ -219,7 +165,7 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
   char *strvalue;
   int min;
   int max;
-  unsigned int state;
+  int state;
   int flags;
   int cflags;
   int total;
@@ -274,7 +220,7 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
       }
       break;
     case DP_S_MIN:
-      if ('0' <= ch && ch <= '9') 
+      if (isdigit((unsigned char)ch)) 
       {
 	min = 10*min + char_to_int (ch);
 	ch = *format++;
@@ -298,7 +244,7 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	state = DP_S_MOD;
       break;
     case DP_S_MAX:
-      if ('0' <= ch && ch <= '9')
+      if (isdigit((unsigned char)ch)) 
       {
 	if (max < 0)
 	  max = 0;
@@ -390,8 +336,6 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	break;
       case 'X':
 	flags |= DP_F_UP;
-        __attribute__((fallthrough));
-        /* fall through */
       case 'x':
 	flags |= DP_F_UNSIGNED;
 	if (cflags == DP_C_SHORT)
@@ -408,38 +352,28 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	if (cflags == DP_C_LDOUBLE)
 	  fvalue = va_arg (args, LDOUBLE);
 	else
-	  fvalue = (LDOUBLE) va_arg (args, double);
+	  fvalue = va_arg (args, double);
+	/* um, floating point? */
 	total += fmtfp (buffer, &currlen, maxlen, fvalue, min, max, flags);
 	break;
       case 'E':
 	flags |= DP_F_UP;
-        __attribute__((fallthrough));
-        /* fall through */
       case 'e':
 	if (cflags == DP_C_LDOUBLE)
 	  fvalue = va_arg (args, LDOUBLE);
 	else
-	  fvalue = (LDOUBLE) va_arg (args, double);
-        total += fmtfp (buffer, &currlen, maxlen, fvalue, min, max, flags);
+	  fvalue = va_arg (args, double);
 	break;
       case 'G':
 	flags |= DP_F_UP;
-        __attribute__((fallthrough));
-        /* fall through */
       case 'g':
-        flags |= DP_F_FP_G;
 	if (cflags == DP_C_LDOUBLE)
 	  fvalue = va_arg (args, LDOUBLE);
 	else
-	  fvalue = (LDOUBLE) va_arg (args, double);
-	if (max == 0)
-	  /* C99 says: if precision [for %g] is zero, it is taken as one */
-	  max = 1;
-	total += fmtfp (buffer, &currlen, maxlen, fvalue, min, max, flags);
+	  fvalue = va_arg (args, double);
 	break;
       case 'c':
-	total += dopr_outch (buffer, &currlen, maxlen,
-			     (char) va_arg (args, int));
+	total += dopr_outch (buffer, &currlen, maxlen, va_arg (args, int));
 	break;
       case 's':
 	strvalue = va_arg (args, char *);
@@ -455,7 +389,7 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	{
 	  short int *num;
 	  num = va_arg (args, short int *);
-	  *num = (short) currlen;
+	  *num = currlen;
         } 
 	else if (cflags == DP_C_LONG) 
 	{
@@ -473,7 +407,7 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	{
 	  int *num;
 	  num = va_arg (args, int *);
-	  *num = (int) currlen;
+	  *num = currlen;
         }
 	break;
       case '%':
@@ -481,7 +415,7 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	break;
       case 'w':
 	/* not supported yet, treat as next char */
-	format++;
+	ch = *format++;
 	break;
       default:
 	/* Unknown, skip */
@@ -510,23 +444,21 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 }
 
 static int fmtstr (char *buffer, size_t *currlen, size_t maxlen,
-                   const char *value, int flags, int min, int max)
+                   char *value, int flags, int min, int max)
 {
   int padlen, strln;     /* amount to pad */
   int cnt = 0;
   int total = 0;
+  char null[] = "<NULL>";
   
   if (value == 0)
   {
-    value = "(null)";
+    value = null;
   }
 
-  if (max < 0)
-    strln = (int) strlen (value);
-  else
-    /* When precision is specified, don't read VALUE past precision. */
-    /*strln = strnlen (value, max);*/
-    for (strln = 0; strln < max && value[strln]; ++strln);
+  for (strln = 0; value[strln]; ++strln); /* strlen */
+  if (max >= 0 && max < strln)
+    strln = max;
   padlen = min - strln;
   if (padlen < 0) 
     padlen = 0;
@@ -556,7 +488,7 @@ static int fmtstr (char *buffer, size_t *currlen, size_t maxlen,
 static int fmtint (char *buffer, size_t *currlen, size_t maxlen,
 		   LLONG value, int base, int min, int max, int flags)
 {
-  char signvalue = 0;
+  int signvalue = 0;
   unsigned LLONG uvalue;
   char convert[24];
   unsigned int place = 0;
@@ -610,8 +542,8 @@ static int fmtint (char *buffer, size_t *currlen, size_t maxlen,
     spadlen = -spadlen; /* Left Justifty */
 
 #ifdef DEBUG_SNPRINTF
-  debug ("zpad: %d, spad: %d, min: %d, max: %d, place: %u\n",
-         zpadlen, spadlen, min, max, place);
+  dprint (1, (debugfile, "zpad: %d, spad: %d, min: %d, max: %d, place: %d\n",
+      zpadlen, spadlen, min, max, place));
 #endif
 
   /* Spaces */
@@ -658,7 +590,7 @@ static LDOUBLE abs_val (LDOUBLE value)
   return result;
 }
 
-static LLONG pow10_int (unsigned int exp)
+static LDOUBLE pow10 (int exp)
 {
   LDOUBLE result = 1;
 
@@ -668,47 +600,36 @@ static LLONG pow10_int (unsigned int exp)
     exp--;
   }
   
-  return (LLONG) result;
+  return result;
 }
 
-static LLONG round_int (LDOUBLE value)
+static long round (LDOUBLE value)
 {
-  LLONG intpart;
+  long intpart;
 
-  intpart = (LLONG) value;
+  intpart = value;
   value = value - intpart;
-  if (value >= (LDOUBLE) 0.5)
+  if (value >= 0.5)
     intpart++;
 
   return intpart;
 }
 
-/*
- * GCC 7.1 issues this warning at the point of the function definition header
- * (not in any actual code), and I can't figure out what's triggering it since
- * the comparison form doesn't appear anywhere in this code.  Since this is
- * rarely-used portability code, suppress the warning.
- */
-#pragma GCC diagnostic ignored "-Wstrict-overflow"
-
 static int fmtfp (char *buffer, size_t *currlen, size_t maxlen,
 		  LDOUBLE fvalue, int min, int max, int flags)
 {
-  char signvalue = 0;
+  int signvalue = 0;
   LDOUBLE ufvalue;
-  char iconvert[24];
-  char fconvert[24];
-  size_t iplace = 0;
-  size_t fplace = 0;
-  long padlen = 0; /* amount to pad */
-  long zpadlen = 0; 
+  char iconvert[20];
+  char fconvert[20];
+  int iplace = 0;
+  int fplace = 0;
+  int padlen = 0; /* amount to pad */
+  int zpadlen = 0; 
+  int caps = 0;
   int total = 0;
   LLONG intpart;
   LLONG fracpart;
-  LLONG mask10;
-  int leadingfrac0s = 0; /* zeroes at the start of fractional part */
-  int omitzeros = 0;
-  size_t omitcount = 0;
   
   /* 
    * AIX manpage says the default is 0, but Solaris says the default
@@ -732,106 +653,51 @@ static int fmtfp (char *buffer, size_t *currlen, size_t maxlen,
   if (flags & DP_F_UP) caps = 1; /* Should characters be upper case? */
 #endif
 
-  intpart = (LLONG) ufvalue;
-
-  /* With %g precision is the number of significant digits, which
-     includes the digits in intpart. */
-  if (flags & DP_F_FP_G)
-    {
-      if (intpart != 0)
-	{
-	  /* For each digit of INTPART, print one less fractional digit. */
-	  LLONG temp;
-	  for (temp = intpart; temp != 0; temp /= 10)
-	    --max;
-	  if (max < 0)
-	    max = 0;
-	}
-      else
-	{
-	  /* For each leading 0 in fractional part, print one more
-	     fractional digit. */
-	  LDOUBLE temp;
-	  if (ufvalue > 0)
-	    for (temp = ufvalue; temp < (LDOUBLE) 0.1; temp *= 10)
-	      ++max;
-	}
-    }
-
-  /* C99: trailing zeros are removed from the fractional portion of the
-     result unless the # flag is specified */
-  if ((flags & DP_F_FP_G) && !(flags & DP_F_NUM))
-    omitzeros = 1;
-
-#if SIZEOF_LONG_LONG > 0
-# define MAX_DIGITS 18		/* grok more digits with long long */
-#else
-# define MAX_DIGITS 9		/* just long */
-#endif
+  intpart = ufvalue;
 
   /* 
-   * Sorry, we only support several digits past the decimal because of
-   * our conversion method
+   * Sorry, we only support 9 digits past the decimal because of our 
+   * conversion method
    */
-  if (max > MAX_DIGITS)
-    max = MAX_DIGITS;
-
-  /* Factor of 10 with the needed number of digits, e.g. 1000 for max==3 */
-  mask10 = pow10_int (max);
+  if (max > 9)
+    max = 9;
 
   /* We "cheat" by converting the fractional part to integer by
    * multiplying by a factor of 10
    */
-  fracpart = round_int (mask10 * (ufvalue - intpart));
+  fracpart = round ((pow10 (max)) * (ufvalue - intpart));
 
-  if (fracpart >= mask10)
+  if (fracpart >= pow10 (max))
   {
     intpart++;
-    fracpart -= mask10;
+    fracpart -= pow10 (max);
   }
-  else if (fracpart != 0)
-    /* If fracpart has less digits than the 10* mask, we need to
-       manually insert leading 0s.  For example 2.01's fractional part
-       requires one leading zero to distinguish it from 2.1. */
-    while (fracpart < mask10 / 10)
-      {
-	++leadingfrac0s;
-	mask10 /= 10;
-      }
 
 #ifdef DEBUG_SNPRINTF
-# ifdef HAVE_LONG_LONG_INT
-  debug ("fmtfp: %Lf =? %lld.%lld\n", fvalue, intpart, fracpart);
-# else
-  debug ("fmtfp: %Lf =? %ld.%ld\n", fvalue, intpart, fracpart);
-# endif
+  dprint (1, (debugfile, "fmtfp: %f =? %d.%d\n", fvalue, intpart, fracpart));
 #endif
 
   /* Convert integer part */
   do {
-    iconvert[iplace++] = (char) ('0' + (intpart % 10));
+    iconvert[iplace++] =
+      (caps? "0123456789ABCDEF":"0123456789abcdef")[intpart % 10];
     intpart = (intpart / 10);
-  } while(intpart && (iplace < sizeof(iconvert)));
-  if (iplace == sizeof(iconvert)) iplace--;
+  } while(intpart && (iplace < 20));
+  if (iplace == 20) iplace--;
   iconvert[iplace] = 0;
 
   /* Convert fractional part */
   do {
-    fconvert[fplace++] = (char) ('0' + (fracpart % 10));
+    fconvert[fplace++] =
+      (caps? "0123456789ABCDEF":"0123456789abcdef")[fracpart % 10];
     fracpart = (fracpart / 10);
-  } while(fracpart && (fplace < sizeof(fconvert)));
-  while (leadingfrac0s-- > 0 && fplace < sizeof(fconvert))
-    fconvert[fplace++] = '0';
-  if (fplace == sizeof(fconvert)) fplace--;
+  } while(fracpart && (fplace < 20));
+  if (fplace == 20) fplace--;
   fconvert[fplace] = 0;
-  if (omitzeros)
-    while (omitcount < fplace && fconvert[omitcount] == '0')
-      ++omitcount;
 
   /* -1 for decimal point, another -1 if we are printing a sign */
-  padlen = min - iplace - (max - omitcount) - 1 - ((signvalue) ? 1 : 0);
-  if (!omitzeros)
-    zpadlen = max - fplace;
+  padlen = min - iplace - max - 1 - ((signvalue) ? 1 : 0); 
+  zpadlen = max - fplace;
   if (zpadlen < 0)
     zpadlen = 0;
   if (padlen < 0) 
@@ -868,11 +734,11 @@ static int fmtfp (char *buffer, size_t *currlen, size_t maxlen,
    * Decimal point.  This should probably use locale to find the correct
    * char to print out.
    */
-  if (max > 0 && (fplace > omitcount || zpadlen > 0))
+  if (max > 0)
   {
     total += dopr_outch (buffer, currlen, maxlen, '.');
 
-    while (fplace > omitcount) 
+    while (fplace > 0) 
       total += dopr_outch (buffer, currlen, maxlen, fconvert[--fplace]);
   }
 
@@ -905,14 +771,27 @@ int vsnprintf (char *str, size_t count, const char *fmt, va_list args)
   return dopr(str, count, fmt, args);
 }
 
-int snprintf (char *str, size_t count, const char *fmt,...)
+/* VARARGS3 */
+#ifdef HAVE_STDARGS
+int snprintf (char *str,size_t count,const char *fmt,...)
+#else
+int snprintf (va_alist) va_dcl
+#endif
 {
-  va_list ap;
+#ifndef HAVE_STDARGS
+  char *str;
+  size_t count;
+  char *fmt;
+#endif
+  VA_LOCAL_DECL;
   int total;
     
-  va_start(ap, fmt);
+  VA_START (fmt);
+  VA_SHIFT (str, char *);
+  VA_SHIFT (count, size_t );
+  VA_SHIFT (fmt, char *);
   total = vsnprintf(str, count, fmt, ap);
-  va_end(ap);
+  VA_END;
   return total;
 }
 
@@ -989,6 +868,5 @@ int main (void)
       num++;
     }
   printf ("%d tests failed out of %d.\n", fail, num);
-  return 0;
 }
-#endif /* TEST_SNPRINTF */
+#endif /* SNPRINTF_TEST */
