@@ -1,12 +1,12 @@
-/*  $Revision: 1.41.2.3 $
+/*  $Id: group.c,v 1.62 2003/02/10 03:28:25 rra Exp $
 **
 **  Newsgroups and the active file.
 */
+
 #include "config.h"
 #include "clibrary.h"
-#include <netinet/in.h>
-#include <sys/mman.h>
 
+#include "inn/innconf.h"
 #include "nnrpd.h"
 #include "ov.h"
 
@@ -14,7 +14,7 @@
 **  Change to or list the specified newsgroup.  If invalid, stay in the old
 **  group.
 */
-FUNCTYPE CMDgroup(int ac, char *av[])
+void CMDgroup(int ac, char *av[])
 {
     static char		NOSUCHGROUP[] = NNTP_NOSUCHGROUP;
     ARTNUM              i;
@@ -23,8 +23,14 @@ FUNCTYPE CMDgroup(int ac, char *av[])
     void                *handle;
     TOKEN               token;
     int                 count;
+    bool		boolval;
+    bool                hookpresent = false;
 
-    if (!PERMcanread) {
+#ifdef DO_PYTHON
+    hookpresent = PY_use_dynamic;
+#endif /* DO_PYTHON */
+
+    if (!hookpresent && !PERMcanread) {
 	Reply("%s\r\n", NOACCESS);
 	return;
     }
@@ -35,30 +41,31 @@ FUNCTYPE CMDgroup(int ac, char *av[])
 	    Printf("%d No group specified\r\n", NNTP_XGTITLE_BAD);
 	    return;
 	} else {
-	    group = COPY(GRPcur);
+	    group = xstrdup(GRPcur);
 	}
     } else {
-	group = COPY(av[1]);
+	group = xstrdup(av[1]);
     }
     
     if (!OVgroupstats(group, &ARTlow, &ARThigh, &count, NULL)) {
 	Reply("%s %s\r\n", NOSUCHGROUP, group);
-	DISPOSE(group);
+	free(group);
 	return;
     }
 
 #ifdef DO_PYTHON
-    if (innconf->nnrppythonauth) {
+    if (PY_use_dynamic) {
         char    *reply;
 
-	/* Authorize user at a Python authorization module */
-	if (PY_authorize(ClientHost, ClientIp, ServerHost, PERMuser, group, FALSE, &reply) < 0) {
-	    syslog(L_NOTICE, "PY_authorize(): authorization skipped due to no Python authorization method defined.");
+	/* Authorize user using Python module method dynamic*/
+	if (PY_dynamic(PERMuser, group, false, &reply) < 0) {
+	    syslog(L_NOTICE, "PY_dynamic(): authorization skipped due to no Python dynamic method defined.");
 	} else {
 	    if (reply != NULL) {
-	        syslog(L_TRACE, "PY_authorize() returned a refuse string for user %s at %s who wants to read %s: %s", PERMuser, ClientHost, group, reply);
+	        syslog(L_TRACE, "PY_dynamic() returned a refuse string for user %s at %s who wants to read %s: %s", PERMuser, ClientHost, group, reply);
 		Reply("%d %s\r\n", NNTP_ACCESS_VAL, reply);
-		DISPOSE(group);
+		free(group);
+                free(reply);
 		return;
 	    }
 	}
@@ -66,59 +73,108 @@ FUNCTYPE CMDgroup(int ac, char *av[])
 #endif /* DO_PYTHON */
 
     /* If permission is denied, pretend group doesn't exist. */
-    if (PERMspecified) {
-	grplist[0] = group;
-	grplist[1] = NULL;
-	if (!PERMmatch(PERMreadlist, grplist)) {
-	    Reply("%s %s\r\n", NOSUCHGROUP, group);
-	    DISPOSE(group);
-	    return;
-	}
-    } else {
-	Reply("%s %s\r\n", NOSUCHGROUP, group);
-	DISPOSE(group);
-	return;
+    if (!hookpresent) {
+        if (PERMspecified) {
+            grplist[0] = group;
+            grplist[1] = NULL;
+            if (!PERMmatch(PERMreadlist, grplist)) {
+	         Reply("%s %s\r\n", NOSUCHGROUP, group);
+                 free(group);
+                 return;
+            }
+        } else {
+            Reply("%s %s\r\n", NOSUCHGROUP, group);
+            free(group);
+            return;
+        }
     }
 
     /* Close out any existing article, report group stats. */
     ARTclose();
     GRPreport();
-    HIScheck();
 
     /* Doing a "group" command? */
-    if (caseEQ(av[0], "group")) {
+    if (strcasecmp(av[0], "group") == 0) {
 	if (count == 0)
 	    Reply("%d 0 0 0 %s\r\n", NNTP_GROUPOK_VAL, group);
-	else
+	else {
+	    /* if we're an NFS reader, check the last nfsreaderdelay
+	     * articles in the group to see if they arrived in the
+	     * last nfsreaderdelay (default 60) seconds. If they did,
+	     * don't report them as we don't want them to appear too
+	     * soon */
+	    if (innconf->nfsreader) {
+		ARTNUM low, prev;
+		time_t now, arrived;
+
+		time(&now);
+		if (ARTlow + innconf->nfsreaderdelay > ARThigh)
+		    low = ARTlow;
+		else
+		    low = ARThigh - innconf->nfsreaderdelay;
+		handle = OVopensearch(group, low, ARThigh);
+		if (!handle) {
+		    Reply("%d group disappeared\r\n", NNTP_TEMPERR_VAL);
+		    free(group);
+		    return;
+		}
+		prev = low;
+		while (OVsearch(handle, &i, NULL, NULL, NULL, &arrived)) {
+		    if (arrived + innconf->nfsreaderdelay > now) {
+			ARThigh = prev;
+			break;
+		    }
+		    prev = i;
+		}
+		OVclosesearch(handle);
+	    }
 	    Reply("%d %d %ld %ld %s\r\n",
 		NNTP_GROUPOK_VAL,
 		count, ARTlow, ARThigh, group);
+	}
 	GRPcount++;
 	ARTnumber = ARTlow;
-	if (GRPcur)
-	    DISPOSE(GRPcur);
-	GRPcur = COPY(group);
+	if (GRPcur) {
+	    if (strcmp(GRPcur, group) != 0) {
+		OVctl(OVCACHEFREE, &boolval);
+		free(GRPcur);
+		GRPcur = xstrdup(group);
+	    }
+	} else
+	    GRPcur = xstrdup(group);
     } else {
-	/* Must be doing a "listgroup" command. */
+	/* Must be doing a "listgroup" command.  We used to just return
+           something bland here ("Article list follows"), but reference NNTP
+           returns the same data as GROUP does and since we have it all
+           available it shouldn't hurt to return the same thing. */
 	if ((handle = OVopensearch(group, ARTlow, ARThigh)) != NULL) {
-	Reply("%d Article list follows\r\n", NNTP_GROUPOK_VAL);
+            if (count == 0)
+                Reply("%d 0 0 0 %s\r\n", NNTP_GROUPOK_VAL, group);
+            else
+                Reply("%d %d %ld %ld %s\r\n", NNTP_GROUPOK_VAL, count,
+                      ARTlow, ARThigh, group);
 	    while (OVsearch(handle, &i, NULL, NULL, &token, NULL)) {
 		if (PERMaccessconf->nnrpdcheckart && !ARTinstorebytoken(token))
 		    continue;
 		Printf("%ld\r\n", i);
 	    }
 	    OVclosesearch(handle);
-	Printf(".\r\n");
+	    Printf(".\r\n");
 	    GRPcount++;
 	    ARTnumber = ARTlow;
-	    if (GRPcur)
-		DISPOSE(GRPcur);
-	    GRPcur = COPY(group);
+	    if (GRPcur) {
+		if (strcmp(GRPcur, group) != 0) {
+		    OVctl(OVCACHEFREE, &boolval);
+		    free(GRPcur);
+		    GRPcur = xstrdup(group);
+		}
+	    } else
+		GRPcur = xstrdup(group);
 	} else {
 	    Reply("%s %s\r\n", NOSUCHGROUP, group);
 	}
     }
-    DISPOSE(group);
+    free(group);
 }
 
 
@@ -132,7 +188,7 @@ GRPreport()
     char		repbuff[1024];
 
     if (GRPcur) {
-	(void)strcpy(buff, GRPcur);
+	strlcpy(buff, GRPcur, sizeof(buff));
 	syslog(L_NOTICE, "%s group %s %ld", ClientHost, buff, GRParticles);
 	GRParticles = 0;
 	repbuff[0]='\0';
@@ -143,15 +199,15 @@ GRPreport()
 /*
 **  Used by ANU-News clients.
 */
-FUNCTYPE
+void
 CMDxgtitle(ac, av)
     int			ac;
     char		*av[];
 {
-    register QIOSTATE	*qp;
-    register char	*line;
-    register char	*p;
-    register char	*q;
+    QIOSTATE	*qp;
+    char	*line;
+    char	*p;
+    char	*q;
     char		*grplist[2];
     char		save;
 
@@ -186,7 +242,7 @@ CMDxgtitle(ac, av)
 	    continue;
 	save = *q;
 	*q = '\0';
-	if (wildmat(line, p)) {
+	if (uwildmat(line, p)) {
 	    if (PERMspecified) {
 		grplist[0] = line;
 		grplist[1] = NULL;
