@@ -17,17 +17,19 @@
 
 */
 
+#include <sys/types.h>
 #include "config.h"
+#include "nnrpd.h"
 
 #ifdef HAVE_SSL
 
 /* System library. */
 
-#include <sys/types.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <syslog.h>
+#include <sys/stat.h>
 #include <sys/uio.h>
 
 /* taken from lib/parsedate.c */
@@ -61,36 +63,25 @@
 #endif
 
 
+#endif
 
-/* OpenSSL library. */
-
-#include <openssl/lhash.h>
-#include <openssl/bn.h>
-#include <openssl/err.h>
-#include <openssl/pem.h>
-#include <openssl/x509.h>
-#include <openssl/ssl.h>
-
-/* Application-specific. */
-
+/* outside the ifdef so `make depend` works even ifndef HAVE_SSL */
 #include "tls.h"
 
+#ifdef HAVE_SSL
+
 /* We must keep some of the info available */
 static const char hexcodes[] = "0123456789ABCDEF";
 
+static bool tls_initialized = false;
+
 static int verify_depth;
 static int verify_error = X509_V_OK;
 static int do_dump = 0;
-static SSL_CTX *ctx = NULL;
+static SSL_CTX *CTX = NULL;
 SSL *tls_conn = NULL;
 
 #define CCERT_BUFSIZ 256
-static char peer_subject[CCERT_BUFSIZ];
-static char peer_issuer[CCERT_BUFSIZ];
-static char peer_CN[CCERT_BUFSIZ];
-static char issuer_CN[CCERT_BUFSIZ];
-static unsigned char md[EVP_MAX_MD_SIZE];
-static char fingerprint[EVP_MAX_MD_SIZE * 3];
 
 int     tls_serverengine = 0;
 int     tls_serveractive = 0;	/* available or not */
@@ -102,7 +93,7 @@ int     tls_clientactive = 0;	/* available or not */
 char   *tls_peer_CN = NULL;
 char   *tls_issuer_CN = NULL;
 
-char   *tls_protocol = NULL;
+const char   *tls_protocol = NULL;
 const char   *tls_cipher_name = NULL;
 int	tls_cipher_usebits = 0;
 int	tls_cipher_algbits = 0;
@@ -117,8 +108,8 @@ int tls_loglevel = 0;
 
 static void apps_ssl_info_callback(SSL * s, int where, int ret)
 {
-    char   *str;
-    int     w;
+    const char  *str;
+    int         w;
 
     if (tls_loglevel==0) return;
 
@@ -152,16 +143,124 @@ static void apps_ssl_info_callback(SSL * s, int where, int ret)
     }
 }
 
-/* taken from OpenSSL apps/s_cb.c */
 
-static RSA *tmp_rsa_cb(SSL * s, int export, int keylength)
+/*
+ *	Hardcoded DH parameter files, from OpenSSL.
+ *	For information on how these files were generated, see
+ *	"Assigned Number for SKIP Protocols" 
+ *	(http://www.skip-vpn.org/spec/numbers.html.
+ */
+static const char file_dh512[] =
+"-----BEGIN DH PARAMETERS-----\n\
+MEYCQQD1Kv884bEpQBgRjXyEpwpy1obEAxnIByl6ypUM2Zafq9AKUJsCRtMIPWak\n\
+XUGfnHy9iUsiGSa6q6Jew1XpKgVfAgEC\n\
+-----END DH PARAMETERS-----\n";
+
+static const char file_dh1024[] =
+"-----BEGIN DH PARAMETERS-----\n\
+MIGHAoGBAPSI/VhOSdvNILSd5JEHNmszbDgNRR0PfIizHHxbLY7288kjwEPwpVsY\n\
+jY67VYy4XTjTNP18F1dDox0YbN4zISy1Kv884bEpQBgRjXyEpwpy1obEAxnIByl6\n\
+ypUM2Zafq9AKUJsCRtMIPWakXUGfnHy9iUsiGSa6q6Jew1XpL3jHAgEC\n\
+-----END DH PARAMETERS-----\n";
+
+static const char file_dh2048[] =
+"-----BEGIN DH PARAMETERS-----\n\
+MIIBCAKCAQEA9kJXtwh/CBdyorrWqULzBej5UxE5T7bxbrlLOCDaAadWoxTpj0BV\n\
+89AHxstDqZSt90xkhkn4DIO9ZekX1KHTUPj1WV/cdlJPPT2N286Z4VeSWc39uK50\n\
+T8X8dryDxUcwYc58yWb/Ffm7/ZFexwGq01uejaClcjrUGvC/RgBYK+X0iP1YTknb\n\
+zSC0neSRBzZrM2w4DUUdD3yIsxx8Wy2O9vPJI8BD8KVbGI2Ou1WMuF040zT9fBdX\n\
+Q6MdGGzeMyEstSr/POGxKUAYEY18hKcKctaGxAMZyAcpesqVDNmWn6vQClCbAkbT\n\
+CD1mpF1Bn5x8vYlLIhkmuquiXsNV6TILOwIBAg==\n\
+-----END DH PARAMETERS-----\n";
+
+static const char file_dh4096[] =
+"-----BEGIN DH PARAMETERS-----\n\
+MIICCAKCAgEA+hRyUsFN4VpJ1O8JLcCo/VWr19k3BCgJ4uk+d+KhehjdRqNDNyOQ\n\
+l/MOyQNQfWXPeGKmOmIig6Ev/nm6Nf9Z2B1h3R4hExf+zTiHnvVPeRBhjdQi81rt\n\
+Xeoh6TNrSBIKIHfUJWBh3va0TxxjQIs6IZOLeVNRLMqzeylWqMf49HsIXqbcokUS\n\
+Vt1BkvLdW48j8PPv5DsKRN3tloTxqDJGo9tKvj1Fuk74A+Xda1kNhB7KFlqMyN98\n\
+VETEJ6c7KpfOo30mnK30wqw3S8OtaIR/maYX72tGOno2ehFDkq3pnPtEbD2CScxc\n\
+alJC+EL7RPk5c/tgeTvCngvc1KZn92Y//EI7G9tPZtylj2b56sHtMftIoYJ9+ODM\n\
+sccD5Piz/rejE3Ome8EOOceUSCYAhXn8b3qvxVI1ddd1pED6FHRhFvLrZxFvBEM9\n\
+ERRMp5QqOaHJkM+Dxv8Cj6MqrCbfC4u+ZErxodzuusgDgvZiLF22uxMZbobFWyte\n\
+OvOzKGtwcTqO/1wV5gKkzu1ZVswVUQd5Gg8lJicwqRWyyNRczDDoG9jVDxmogKTH\n\
+AaqLulO7R8Ifa1SwF2DteSGVtgWEN8gDpN3RBmmPTDngyF2DHb5qmpnznwtFKdTL\n\
+KWbuHn491xNO25CQWMtem80uKw+pTnisBRF/454n1Jnhub144YRBoN8CAQI=\n\
+-----END DH PARAMETERS-----\n";
+
+/*
+ *	Load hardcoded DH parameters.
+ */
+static DH *
+load_dh_buffer (const char *buffer, size_t len)
 {
-    static RSA *rsa_tmp = NULL;
+	BIO *bio;
+	DH *dh = NULL;
 
-    if (rsa_tmp == NULL) {
-	rsa_tmp = RSA_generate_key(keylength, RSA_F4, NULL, NULL);
-    }
-    return (rsa_tmp);
+	bio = BIO_new_mem_buf((char *) buffer, len);
+	if (bio == NULL)
+		return NULL;
+	dh = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
+/*	if (dh == NULL) log error */
+	BIO_free(bio);
+
+	return dh;
+}
+
+/*
+ *	Generate empheral DH key.  Because this can take a long
+ *	time to compute, we use precomputed parameters of the
+ *	common key sizes.
+ *
+ *	These values can be static (once loaded or computed) since
+ *	the OpenSSL library can effectively generate random keys
+ *	from the information provided.
+ *
+ *	EDH keying is slightly less efficient than static RSA keying,
+ *	but it offers Perfect Forward Secrecy (PFS).
+ *
+ *	FIXME: support user-specified files, to eliminate risk of
+ *	"small group" attacks.
+ */
+static DH *tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
+{
+	DH *r = NULL;
+	static DH *dh = NULL;
+	static DH *dh512 = NULL;
+	static DH *dh1024 = NULL;
+	static DH *dh2048 = NULL;
+	static DH *dh4096 = NULL;
+
+	switch (keylength)
+	{
+	case 512:
+		if (dh512 == NULL)
+			dh512 = load_dh_buffer(file_dh512, sizeof file_dh512);
+		r = dh512;
+		break;
+	case 1024:
+		if (dh1024 == NULL)
+			dh1024 = load_dh_buffer(file_dh1024, sizeof file_dh1024);
+		r = dh1024;
+		break;
+	case 2048:
+		if (dh2048 == NULL)
+			dh2048 = load_dh_buffer(file_dh2048, sizeof file_dh2048);
+		r = dh2048;
+		break;
+	case 4096:
+		if (dh4096 == NULL)
+			dh4096 = load_dh_buffer(file_dh4096, sizeof file_dh4096);
+		r = dh4096;
+		break;
+	default:
+		/* we should check current keylength vs. requested keylength */
+		/* also, this is an extremely expensive operation! */
+		dh = DH_generate_parameters(keylength, DH_GENERATOR_2, NULL, NULL);
+		r = dh;
+	}
+
+	return r;
 }
 
 /* taken from OpenSSL apps/s_cb.c */
@@ -257,7 +356,7 @@ static int tls_dump(const char *s, int len)
 	    if (((i * DUMP_WIDTH) + j) >= len) {
 		strcpy(ss, "   ");
 	    } else {
-		ch = ((unsigned char) *((char *) (s) + i * DUMP_WIDTH + j))
+		ch = ((unsigned char) *((const char *)(s) + i * DUMP_WIDTH + j))
 		    & 0xff;
 		sprintf(ss, "%02x%c", ch, j == 7 ? '|' : ' ');
 		ss += 3;
@@ -268,7 +367,8 @@ static int tls_dump(const char *s, int len)
 	for (j = 0; j < DUMP_WIDTH; j++) {
 	    if (((i * DUMP_WIDTH) + j) >= len)
 		break;
-	    ch = ((unsigned char) *((char *) (s) + i * DUMP_WIDTH + j)) & 0xff;
+	    ch = ((unsigned char) *((const char *)(s) + i * DUMP_WIDTH + j))
+		& 0xff;
 	    *ss+= (((ch >= ' ') && (ch <= '~')) ? ch : '.');
 	    if (j == 7) *ss+= ' ';
 	}
@@ -283,7 +383,7 @@ static int tls_dump(const char *s, int len)
     }
 #ifdef TRUNCATE
     if (trunc > 0) {
-	sprintf(buf, "%04x - <SPACES/NULS>\n", len+ trunc);
+	snprintf(buf, sizeof(buf), "%04x - <SPACES/NULS>\n", len+ trunc);
 	if (tls_loglevel>0)
 	  Printf("%s", buf);
 	ret += strlen(buf);
@@ -302,6 +402,8 @@ static int tls_dump(const char *s, int len)
 
 static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 {
+    struct stat buf;
+
     if (cert_file != NULL) {
 	if (SSL_CTX_use_certificate_file(ctx, cert_file,
 					 SSL_FILETYPE_PEM) <= 0) {
@@ -310,6 +412,20 @@ static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 	}
 	if (key_file == NULL)
 	    key_file = cert_file;
+
+	/* check ownership and permissions of key file */
+	if (lstat(key_file, &buf) == -1) {
+	    syslog(L_ERROR, "unable to stat private key '%s'", key_file);
+	    return (0);
+	}
+	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0077) != 0 ||
+	    buf.st_uid != getuid()) {
+	    syslog(L_ERROR, "bad ownership or permissions on private key"
+                   " '%s': private key must be mode 600 and owned by "
+                   NEWSUSER, cert_file);
+	    return (0);
+	}
+
 	if (SSL_CTX_use_PrivateKey_file(ctx, key_file,
 					SSL_FILETYPE_PEM) <= 0) {
 	    syslog(L_ERROR, "unable to get private key from '%s'", key_file);
@@ -351,6 +467,7 @@ int tls_init_serverengine(int verifydepth,
     char   *CAfile;
     char   *s_cert_file;
     char   *s_key_file;
+    struct stat buf;
 
     if (tls_serverengine)
       return (0);				/* already running */
@@ -361,15 +478,15 @@ int tls_init_serverengine(int verifydepth,
     SSL_load_error_strings();
     SSLeay_add_ssl_algorithms();
 
-    ctx = SSL_CTX_new(SSLv23_server_method());
-    if (ctx == NULL) {
+    CTX = SSL_CTX_new(SSLv23_server_method());
+    if (CTX == NULL) {
       return (-1);
     };
 
     off |= SSL_OP_ALL;		/* Work around all known bugs */
-    SSL_CTX_set_options(ctx, off);
-    SSL_CTX_set_info_callback(ctx, apps_ssl_info_callback);
-    SSL_CTX_sess_set_cache_size(ctx, 128);
+    SSL_CTX_set_options(CTX, off);
+    SSL_CTX_set_info_callback(CTX, apps_ssl_info_callback);
+    SSL_CTX_sess_set_cache_size(CTX, 128);
 
     if (strlen(tls_CAfile) == 0)
       CAfile = NULL;
@@ -380,8 +497,8 @@ int tls_init_serverengine(int verifydepth,
     else
       CApath = tls_CApath;
 
-    if ((!SSL_CTX_load_verify_locations(ctx, CAfile, CApath)) ||
-	(!SSL_CTX_set_default_verify_paths(ctx))) {
+    if ((!SSL_CTX_load_verify_locations(CTX, CAfile, CApath)) ||
+	(!SSL_CTX_set_default_verify_paths(CTX))) {
       if (tls_loglevel >= 2)
 	Printf("TLS engine: cannot load CA data\n");
       return (-1);
@@ -396,12 +513,19 @@ int tls_init_serverengine(int verifydepth,
     else
       s_key_file = tls_key_file;
     
-    if (!set_cert_stuff(ctx, s_cert_file, s_key_file)) {
+    if (!set_cert_stuff(CTX, s_cert_file, s_key_file)) {
       if (tls_loglevel >= 2)
 	Printf("TLS engine: cannot load cert/key data\n");
       return (-1);
     }
-    SSL_CTX_set_tmp_rsa_callback(ctx, tmp_rsa_cb);
+
+    /* load some randomization data from /dev/urandom, if it exists */
+    /* FIXME: should also check for ".rand" file, update it on exit */
+    if (stat("/dev/urandom", &buf) == 0)
+      RAND_load_file("/dev/urandom", 16 * 1024);
+
+    SSL_CTX_set_tmp_dh_callback(CTX, tmp_dh_cb);
+    SSL_CTX_set_options(CTX, SSL_OP_SINGLE_DH_USE);
 
     verify_depth = verifydepth;
     if (askcert!=0)
@@ -409,19 +533,53 @@ int tls_init_serverengine(int verifydepth,
     if (requirecert)
 	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT
 	    | SSL_VERIFY_CLIENT_ONCE;
-    SSL_CTX_set_verify(ctx, verify_flags, verify_callback);
+    SSL_CTX_set_verify(CTX, verify_flags, verify_callback);
 
-    SSL_CTX_set_client_CA_list(ctx, SSL_load_client_CA_file(CAfile));
+    SSL_CTX_set_client_CA_list(CTX, SSL_load_client_CA_file(CAfile));
 
     tls_serverengine = 1;
     return (0);
 }
 
 
+/*
+**  The function called by nnrpd to initialize the TLS support.  Calls
+**  tls_init_server_engine and checks the result.  On any sort of failure,
+**  nnrpd will exit.
+*/
+void
+tls_init(void)
+{
+    int ssl_result;
+
+    if (tls_initialized)
+        return;
+    sasl_config_read();
+    ssl_result = tls_init_serverengine(5,        /* depth to verify */
+				       0,        /* can client auth? */
+				       0,        /* required client to auth? */
+				       (char *)sasl_config_getstring("tls_ca_file", ""),
+				       (char *)sasl_config_getstring("tls_ca_path", ""),
+				       (char *)sasl_config_getstring("tls_cert_file", ""),
+				       (char *)sasl_config_getstring("tls_key_file", ""));
+    if (ssl_result == -1) {
+        Reply("%d Error initializing TLS\r\n", NNTP_STARTTLS_BAD_VAL);
+        syslog(L_ERROR, "error initializing TLS: "
+               "[CA_file: %s] [CA_path: %s] [cert_file: %s] [key_file: %s]",
+               sasl_config_getstring("tls_ca_file", ""),
+               sasl_config_getstring("tls_ca_path", ""),
+               sasl_config_getstring("tls_cert_file", ""),
+               sasl_config_getstring("tls_key_file", ""));
+        ExitWithStats(1, false);
+    }
+    tls_initialized = true;
+}
+
+
 /* taken from OpenSSL apps/s_cb.c */
 
 static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
-			long argl, long ret)
+			long argl UNUSED, long ret)
 {
     if (!do_dump)
 	return (ret);
@@ -452,8 +610,6 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
 int tls_start_servertls(int readfd, int writefd)
 {
     int     sts;
-    int     j;
-    unsigned int n;
     SSL_SESSION *session;
     SSL_CIPHER *cipher;
 
@@ -468,7 +624,7 @@ int tls_start_servertls(int readfd, int writefd)
 
     if (tls_conn == NULL)
     {
-	tls_conn = (SSL *) SSL_new(ctx);
+	tls_conn = (SSL *) SSL_new(CTX);
     }
     if (tls_conn == NULL)
     {
@@ -488,7 +644,6 @@ int tls_start_servertls(int readfd, int writefd)
       return (-1);
     }
     
-
     /*
      * This is the actual handshake routine. It will do all the negotiations
      * and will check the client cert etc.
@@ -511,7 +666,7 @@ int tls_start_servertls(int readfd, int writefd)
 	session = SSL_get_session(tls_conn);
 
 	if (session) {
-	  SSL_CTX_remove_session(ctx, session);
+	  SSL_CTX_remove_session(CTX, session);
 	}
 	if (tls_conn)
 	  SSL_free(tls_conn);
@@ -543,7 +698,7 @@ SSL_writev (ssl, vector, count)
      int count;
 {
   char *buffer;
-  register char *bp;
+  char *bp;
   size_t bytes, to_copy;
   int i;
   /* Find the total number of bytes to be written.  */
@@ -559,7 +714,7 @@ SSL_writev (ssl, vector, count)
     {
 #define min(a, b)       ((a) > (b) ? (b) : (a))
       size_t copy = min (vector[i].iov_len, to_copy);
-      (void) memcpy ((void *) bp, (void *) vector[i].iov_base, copy);
+      memcpy (bp, vector[i].iov_base, copy);
       bp += copy;
       to_copy -= copy;
       if (to_copy == 0)
